/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef DPC_IMPL_TYPE_WPEXTENDLANESTRUCT_H
#define DPC_IMPL_TYPE_WPEXTENDLANESTRUCT_H

#include <cfloat>
#include <cmath>
#include "impl_type_uint8.h"
#include "dpc/impl_type_array_uint8_t_3.h"
#include "impl_type_float.h"
#include "dpc/impl_type_vector_adjacentextendlanestruct.h"
#include "dpc/impl_type_vector_smastruct.h"

namespace dpc {
struct WPExtendLaneStruct {
    ::UInt8 lane_id{ 0U };
    ::dpc::array_uint8_t_3 padding{};
    ::Float lane_latest_lc_point_offset_to_wp{ 0.0F };
    ::dpc::vector_AdjacentExtendLaneStruct left_extend_lane;
    ::dpc::vector_AdjacentExtendLaneStruct right_extend_lane;
    ::UInt8 lane_priority{ 0U };
    ::UInt8 is_emergency_lane{ 0U };
    ::Float current_extendlane_offset{ 0.0F };
    ::Float current_extendlane_endoffset{ 0.0F };
    ::dpc::vector_SMAStruct sma_info;

    static bool IsPlane()
    {
        return false;
    }


    using IsEnumerableTag = void;
    template<typename F>
    void enumerate(F& fun)
    {
        fun(lane_id);
        fun(padding);
        fun(lane_latest_lc_point_offset_to_wp);
        fun(left_extend_lane);
        fun(right_extend_lane);
        fun(lane_priority);
        fun(is_emergency_lane);
        fun(current_extendlane_offset);
        fun(current_extendlane_endoffset);
        fun(sma_info);
    }

    template<typename F>
    void enumerate(F& fun) const
    {
        fun(lane_id);
        fun(padding);
        fun(lane_latest_lc_point_offset_to_wp);
        fun(left_extend_lane);
        fun(right_extend_lane);
        fun(lane_priority);
        fun(is_emergency_lane);
        fun(current_extendlane_offset);
        fun(current_extendlane_endoffset);
        fun(sma_info);
    }

    template<typename F>
    void enumerate_internal(F& fun)
    {
        fun("lane_id", lane_id);
        fun("padding", padding);
        fun("lane_latest_lc_point_offset_to_wp", lane_latest_lc_point_offset_to_wp);
        fun("left_extend_lane", left_extend_lane);
        fun("right_extend_lane", right_extend_lane);
        fun("lane_priority", lane_priority);
        fun("is_emergency_lane", is_emergency_lane);
        fun("current_extendlane_offset", current_extendlane_offset);
        fun("current_extendlane_endoffset", current_extendlane_endoffset);
        fun("sma_info", sma_info);
    }

    template<typename F>
    void enumerate_internal(F& fun) const
    {
        fun("lane_id", lane_id);
        fun("padding", padding);
        fun("lane_latest_lc_point_offset_to_wp", lane_latest_lc_point_offset_to_wp);
        fun("left_extend_lane", left_extend_lane);
        fun("right_extend_lane", right_extend_lane);
        fun("lane_priority", lane_priority);
        fun("is_emergency_lane", is_emergency_lane);
        fun("current_extendlane_offset", current_extendlane_offset);
        fun("current_extendlane_endoffset", current_extendlane_endoffset);
        fun("sma_info", sma_info);
    }

    friend bool operator==(const ::dpc::WPExtendLaneStruct& lhs, const ::dpc::WPExtendLaneStruct& rhs) noexcept
    {
        return (lhs.lane_id == rhs.lane_id) && (lhs.padding == rhs.padding) && (fabs(static_cast<double>(lhs.lane_latest_lc_point_offset_to_wp - rhs.lane_latest_lc_point_offset_to_wp)) < DBL_EPSILON) && (lhs.left_extend_lane == rhs.left_extend_lane) && (lhs.right_extend_lane == rhs.right_extend_lane) && (lhs.lane_priority == rhs.lane_priority) && (lhs.is_emergency_lane == rhs.is_emergency_lane) && (fabs(static_cast<double>(lhs.current_extendlane_offset - rhs.current_extendlane_offset)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.current_extendlane_endoffset - rhs.current_extendlane_endoffset)) < DBL_EPSILON) && (lhs.sma_info == rhs.sma_info);
    }
};
} // namespace dpc


#endif // DPC_IMPL_TYPE_WPEXTENDLANESTRUCT_H
